package com.hidglobal.omnikeydemo

import android.os.AsyncTask
import android.smartcardio.Card
import android.smartcardio.CardException
import android.smartcardio.CardTerminal
import com.hidglobal.omnikeydemo.Constants.WILDCARD_PROTOCOL
import com.hidglobal.omnikeydemo.extensions.sendAPDU
import com.hidglobal.omnikeydemo.model.CardCommunicationProgress
import com.hidglobal.omnikeydemo.model.CardCommunicationStep
import timber.log.Timber
import java.lang.NumberFormatException
import java.lang.ref.WeakReference

internal class GetApduResponseTask(demoActivity: OmniKeyDemoActivity) : AsyncTask<CardTerminal, CardCommunicationProgress, Void>() {

    private val demoActivityRef: WeakReference<OmniKeyDemoActivity> = WeakReference(demoActivity)

    override fun doInBackground(vararg params: CardTerminal): Void? {
        val terminal = params[0]
        try {
            publishProgress(CardCommunicationProgress(CardCommunicationStep.CLEAR_UI))
            if (terminal.isCardPresent) {
                Timber.d("Card present on terminal %s", terminal.name)
                communicateWithCardAndUpdateUi(terminal)
                Timber.d("Card can be removed from terminal %s", terminal.name)
            } else {
                publishProgress(CardCommunicationProgress(CardCommunicationStep.CARD_STATUS, demoActivityRef.get()?.getString(R.string.no_card_detected).orEmpty()))
            }
        } catch (e: Exception) {
            Timber.e(e)
        }
        return null
    }

    public override fun onProgressUpdate(vararg params: CardCommunicationProgress) {
        if (params.isNotEmpty()) {
            val cp = params[0]
            when (cp.step) {
                CardCommunicationStep.CLEAR_UI -> demoActivityRef.get()?.clearUI()
                CardCommunicationStep.CARD_STATUS -> demoActivityRef.get()?.setCardStatusText(cp.message)
                CardCommunicationStep.APDU_RESPONSE -> demoActivityRef.get()?.setCardResponseText(cp.message)
            }
        }
    }

    @Throws(CardException::class)
    private fun communicateWithCardAndUpdateUi(reader: CardTerminal) {
        publishProgress(CardCommunicationProgress(CardCommunicationStep.CARD_STATUS, demoActivityRef.get()?.getString(R.string.card_present).orEmpty()))
        val card = reader.connect(WILDCARD_PROTOCOL)
        publishProgress(CardCommunicationProgress(CardCommunicationStep.CARD_STATUS, demoActivityRef.get()?.getString(R.string.getting_data).orEmpty()))
        publishProgress(CardCommunicationProgress(CardCommunicationStep.APDU_RESPONSE, getApduResponse(card).toString()))
        publishProgress(CardCommunicationProgress(CardCommunicationStep.CARD_STATUS))
        card.disconnect(true)
    }

    public override fun onCancelled(unused: Void?) {
        demoActivityRef.get()?.clearUI()
    }

    private fun getApduResponse(card: Card): String? {
        var response = ""
        val request = demoActivityRef.get()?.getApduRequestText()
        request?.let {
            try {
                response = ConvertionUtils.byteArrayToString(card.sendAPDU(it), 0)
            } catch (e: NumberFormatException) {
                response = demoActivityRef.get()?.getString(R.string.invalid_apdu).toString()
                Timber.e(e)
            } catch (l: IllegalArgumentException) {
                response = demoActivityRef.get()?.getString(R.string.invalid_apdu_length).toString()
                Timber.e(l)
            }
        }
        return response
    }
}
